# -*- coding=utf-8 -*-
from typing import Dict
import re

from . import data
from .exception import InvalidCharacterException, InvalidTelephoneNumberException

number_and_hyphen_regexp = re.compile(r'^[−ー0-9０-９-]+$')
telephone_number_regexp = re.compile(r'^[0-9０-９]{10,11}$')  # ハイフン無し
telephone_number_with_hyphen_regex = re.compile(r'^[0-9０-９]{2,4}[−ー-][0-9０-９]{2,4}[−ー-][0-9０-９]{3,4}$')  # ハイフン有り

_translate_table = str.maketrans({
    '０': '0',
    '１': '1',
    '２': '2',
    '３': '3',
    '４': '4',
    '５': '5',
    '６': '6',
    '７': '7',
    '８': '8',
    '９': '9',
    'ー': '-',
    '−': '-',
})


def generate_dict(number: str, area_code_length: int, city_code_length: int) -> Dict[str, str]:
    total_code_length = area_code_length + city_code_length
    if len(number) < total_code_length:
        raise InvalidTelephoneNumberException

    return {
        'area_code': number[:area_code_length],
        'city_code': number[area_code_length:total_code_length],
        'subscriber_code': number[total_code_length:],
    }


def zenkaku_to_hankaku(number: str) -> str:
    return number.translate(_translate_table)


def extract_number(src: str) -> str:
    """全角半角やハイフンが混じった文字列を半角数字のみの文字列にして返す"""
    m = number_and_hyphen_regexp.fullmatch(src)
    if not m:
        raise InvalidCharacterException
    return zenkaku_to_hankaku(m.string).replace('-', '')


def split(src: str) -> Dict[str, str]:
    """入力された文字列を辞書形式に分割して返す"""
    number = extract_number(src)
    # 固定電話
    for codes in [data.area_code_5, data.area_code_4, data.area_code_3, data.area_code_2]:
        for code in codes:
            if number.startswith(code):
                area_code_length = len(code)
                city_code_length = data.TOTAL_CODE_LENGTH - area_code_length
                return generate_dict(number, area_code_length, city_code_length)
    # フリーダイヤル
    for code in data.freedial_code:
        if number.startswith(code):
            return generate_dict(number, data.FREEDIAL_CODE_PREFIX_LENGTH, data.FREEDIAL_CODE_LENGTH)

    # 携帯電話
    for code in data.mobile_code:
        if number.startswith(code):
            return generate_dict(number, data.MOBILE_CODE_PREFIX_LENGTH, data.MOBILE_CODE_LENGTH)

    # その他番号
    for code in data.other_code:
        if number.startswith(code):
            return generate_dict(number, data.OTHER_CODE_PREFIX_LENGTH, data.OTHER_CODE_LENGTH)

    raise InvalidTelephoneNumberException


def validate(src: str) -> bool:
    if not telephone_number_regexp.fullmatch(src) and not telephone_number_with_hyphen_regex.fullmatch(src):
        return False
    if not split(src):
        return False
    return True


def normalize(src: str) -> str:
    """入力された文字列をハイフン区切りの電話番号にして返す"""
    number = extract_number(src)
    code = split(number)

    return '{}-{}-{}'.format(code['area_code'], code['city_code'], code['subscriber_code'])



from .area_code import (
    area_code_2,
    area_code_3,
    area_code_4,
    area_code_5,
)

from .defined_code import (
    TOTAL_CODE_LENGTH,
    MOBILE_CODE_PREFIX_LENGTH,
    MOBILE_CODE_LENGTH,
    FREEDIAL_CODE_PREFIX_LENGTH,
    FREEDIAL_CODE_LENGTH,
    OTHER_CODE_PREFIX_LENGTH,
    OTHER_CODE_LENGTH,

    mobile_code,
    freedial_code,
    other_code,
)

__all__ = [
    area_code_2,
    area_code_3,
    area_code_4,
    area_code_5,

    TOTAL_CODE_LENGTH,
    MOBILE_CODE_PREFIX_LENGTH,
    MOBILE_CODE_LENGTH,
    FREEDIAL_CODE_PREFIX_LENGTH,
    FREEDIAL_CODE_LENGTH,
    OTHER_CODE_PREFIX_LENGTH,
    OTHER_CODE_LENGTH,

    mobile_code,
    freedial_code,
    other_code,
]



# -*- coding=utf-8 -*-
from jinja2 import Environment, PackageLoader
import requests
import xlrd

xls_filenames = [
    '000124070.xls',
    '000124071.xls',
    '000124072.xls',
    '000124073.xls',
    '000124074.xls',
    '000124075.xls',
    '000124076.xls',
    '000124077.xls',
    '000124078.xls',
]

master_data_filename = 'area_code.py'
base_url = 'http://www.soumu.go.jp/main_content/'
sheet_name = '公開データ'
data_start_row = 2  # 実際のデータが開始する行数
row_area_code = 3  # 市外局番が格納されたCol数
area_code_dict = {2: [], 3: [], 4: [], 5: []}


def xls_download_from_MIS():
    for filename in xls_filenames:
        print('download {}'.format(filename))
        resp = requests.get(base_url + filename)
        if resp.status_code != 200:
            raise Exception('Excelファイルのダウンロードに失敗しました')
        with open(filename, 'wb') as f:
            f.write(resp.content)


def read_xls_file(filename):
    print('reading {}'.format(filename))
    book = xlrd.open_workbook(filename)
    sheet = book.sheet_by_name(sheet_name)
    for index, row in enumerate(sheet.get_rows()):
        if index < data_start_row:
            continue
        area_code = row[row_area_code].value
        area_code_list = area_code_dict[len(area_code)]
        if not area_code in area_code_list:
            area_code_list.append(area_code)


def generate_master_data():
    print('generating {}'.format(master_data_filename))
    env = Environment(
        loader=PackageLoader('__main__', '')
    )
    tmpl = env.get_template('master_data.tmpl')
    result = tmpl.render(area_code_dict=area_code_dict)
    with open(master_data_filename, 'w') as f:
        f.write(result)


def main():
    # xls_download_from_MIS()
    for filename in xls_filenames:
        read_xls_file(filename)
    generate_master_data()


if __name__ == '__main__':
    main()



TOTAL_CODE_LENGTH = 6
MOBILE_CODE_PREFIX_LENGTH = 3
MOBILE_CODE_LENGTH = 4
FREEDIAL_CODE_PREFIX_LENGTH = 4
FREEDIAL_CODE_LENGTH = 3
OTHER_CODE_PREFIX_LENGTH = 4
OTHER_CODE_LENGTH = 3

mobile_code = [
    '050',  # IP電話
    '070',  # 携帯電話/PHS
    '080',  # 携帯電話
    '090',  # 携帯電話
    '020',  # その他
]

freedial_code = [
    '0120',  # フリーダイヤル 0120-xxx-xxx
    '0800',  # フリーダイヤル 0800-xxx-xxxx
]

other_code = [
    '0570',  # ナビダイヤル 0570-xxx-xxx
    '0990',  # ダイヤルQ2 0990-xxx-xxx
]



# -*- coding=utf-8 -*-
from setuptools import setup, find_packages
import os


long_description = 'Japanese Telephone Number Utility'
if os.path.exists('README.rst'):
    long_description = open('README.rst').read()

setup(
    name='jptel',
    version='0.1.0',
    description='Japanese Telephone Number Utility',
    long_description=long_description,
    license='MIT',
    author='zenwerk',
    author_email='zenwerk@gmail.com',
    url='https://github.com/zenwerk/jptel-py',
    keywords='japaneses telephone number',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3.7',
        'License :: OSI Approved :: MIT License',
    ]
)
